/**
 * \file vm_shutdown.c
 * \brief CKPM virtual machine shutdown functionality
 */
/* Copyright (C) 2006 by Clemens Krainer
 * @(#) $Id:$
 *
 * Author: Clemens Krainer <ClemensDaniel.Krainer@sbg.ac.at>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "vm.h"


/**
 * \brief shut down the virtual machine
 * \param verbose if != 0 this function will provide verbose output
 * \param vm_memory the pointer to the virtual machine memory
 *
 * This function shuts down the virtual machine. Actually, this function does
 * nothing.
 *****************************************************************************/

void
shutdown_vm (int verbose, vm_mem_t* vm_memory) {

   if (verbose)
      fprintf (stderr, "Virtual Machine Shutdown.\n");

   if (vm_memory) {
      free (vm_memory);

      if (verbose)
         fprintf (stderr, "Virtual Machine Memory freed.\n");
   }
}

